/*
 * File Name: ipc.c
 */

/*
 * This file is part of hello_world.
 *
 * hello_world is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * hello_world is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Copyright (C) 2009 iRex Technologies B.V.
 * All rights reserved.
 */

//----------------------------------------------------------------------------
// Include Files
//----------------------------------------------------------------------------

#include "config.h"

// system include files, between < >
#include <stdio.h>
#include <stdlib.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <unistd.h>

// ereader include files, between < >
#include <liberipc/eripc.h>
#include <liberipc/eripc_support.h>
#include <liberutils/er_error.h>

#include "ipc.h"
#include "menu.h"
#include "dr-file.h"

#include "xournal.h"

extern GtkWidget *winMain;
extern GtkWidget *buttonPen;
extern GtkWidget *buttonEraser;

//----------------------------------------------------------------------------
// Type Declarations
//----------------------------------------------------------------------------

typedef struct 
{
    eripc_handler_t *handler;
    const gchar     *message_name;
    const gchar     *interface;
    gint            handler_id;
} functionEntry;


//----------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------

// IPC application
// TODO: set DBUS_APPL_NAME to your application name, use lower-case and digits only
//       or set to PACKAGENAME when this is lower-case and digits only
#define DBUS_APPL_NAME                  "xournal"            // lower-case and digits only
#define DBUS_SERVICE                     "com.irexnet." DBUS_APPL_NAME
#define DBUS_PATH                       "/com/irexnet/" DBUS_APPL_NAME
#define DBUS_INTERFACE                   "com.irexnet." DBUS_APPL_NAME


static const gint IPC_TIMEOUT_DEFAULT  = 5;

// IPC system control
#define DBUS_SERVICE_SYSTEM_CONTROL     "com.irexnet.sysd"

// IPC popup menu
#define DBUS_SERVICE_POPUP_MENU         "com.irexnet.popupmenu"

// IPC Content Browser control
#define DBUS_SERVICE_CTB                "com.irexnet.ctb"


//----------------------------------------------------------------------------
// Static Variables
//----------------------------------------------------------------------------

static eripc_context_t  *eripcContext = NULL;
static int g_xid = 0;


//============================================================================
// Local Function Definitions
//============================================================================

static void on_menu_item         ( eripc_context_t          *context,
                                   const eripc_event_info_t *info,
                                   void                     *user_data );
                                 
static void on_window_activated  ( eripc_context_t          *context,
                                   const eripc_event_info_t *info,
                                   void                     *user_data );
                                 
static void on_window_deactivated( eripc_context_t          *context,
                                   const eripc_event_info_t *info,
                                   void                     *user_data );
                                 
static void on_prepare_unmount   ( eripc_context_t          *context,
                                   const eripc_event_info_t *info,
                                   void                     *user_data );
                                 
static void on_unmounted         ( eripc_context_t          *context,
                                   const eripc_event_info_t *info,
                                   void                     *user_data );

static void on_status_item_activated    (eripc_context_t *context, 
										 const eripc_event_info_t *info, 
										 void *user_data);

static void on_changed_orientation( eripc_context_t          *context,
                                    const eripc_event_info_t *info,
                                    void                     *user_data );

// Exported DBUS API list
static functionEntry    service_functions[] =
        {
            // message handlers (method calls to this service)
            { on_menu_item,             "menuItemActivated",      NULL                        },
            { on_window_activated,      "activatedWindow",        NULL                        },
            { on_window_deactivated,    "deactivatedWindow",      NULL                        },
            { on_status_item_activated, "statusItemActivated",    NULL                        },
            // signal handlers (broadcasted from given service)
            { on_prepare_unmount,       "sysPrepareUnmount",      DBUS_SERVICE_SYSTEM_CONTROL },
            { on_unmounted,             "sysVolumeUnmounted",     DBUS_SERVICE_SYSTEM_CONTROL },
            { on_changed_orientation,   "sysChangedOrientation",  DBUS_SERVICE_SYSTEM_CONTROL },
            { NULL }
        };


//============================================================================
// Functions Implementation
//============================================================================


//----------------------------------------------------------------------------
// Generic
//----------------------------------------------------------------------------

// initialise
void ipc_set_services (void)
{
    gint          i;
    eripc_error_t retval;

	g_assert(eripcContext == NULL);

	//printf("ipc_set_services\n");
	// initialise IPC context
	eripcContext = eripc_init(DBUS_APPL_NAME, "1.0", NULL);
	if (eripcContext == NULL) 
	{
		printf("Failed to initialize eripc context\n");
		exit(1);
	}

    // register message/signal handlers
    for (i = 0 ; service_functions[i].handler != NULL ; i++)
    {
        if (service_functions[i].interface)
        {
            // install signal handler
            retval = eripc_set_signal_handler( eripcContext,
                                               service_functions[i].handler,
                                               NULL,
                                               ERIPC_BUS_SESSION,
                                               service_functions[i].interface,
                                               service_functions[i].message_name,
                                               &(service_functions[i].handler_id) );
            if (retval != ERIPC_ERROR_SUCCESS) 
            {
                printf( "eripc_set_signal_handler [%s] returns [%d]",
                             service_functions[i].message_name,
                             retval );
            }
        }
        else
        {
            // install message handler
            retval = eripc_set_message_handler( eripcContext,
                                                service_functions[i].handler,
                                                NULL,
                                                ERIPC_BUS_SESSION,
                                                DBUS_INTERFACE,
                                                service_functions[i].message_name,
                                                &(service_functions[i].handler_id) );
              
            if (retval != ERIPC_ERROR_SUCCESS) 
            {
                printf( "eripc_set_message_handler [%s] returns [%d]",
                             service_functions[i].message_name,
                             retval );
            }
        }
    }
}


// un-initialise
void ipc_unset_services (void)
{
    gint i;

    for (i = 0 ; service_functions[i].handler_id != 0 ; i++)
    {
        eripc_unset_handler(eripcContext, service_functions[i].handler_id);
    }    
}


//----------------------------------------------------------------------------
// System control
//----------------------------------------------------------------------------

// report "application started"
void ipc_sys_startup_complete ( void )
{
    eripc_error_t   result;

    // parameters for ipc message
    const int       my_pid = getpid();
    const int       my_xid = GDK_WINDOW_XID(winMain->window);
    const gboolean  is_multidoc = FALSE;        // TODO: set TRUE/FALSE depending on your application
	const gboolean  show_taskbar = TRUE;

    g_assert( eripcContext );

	//printf("ipc_sys_startup_complete:%x\n", pub->mw->window->window);
    // broadcast signal over session bus
	g_xid = my_xid;
    result = eripc_send_signal_varargs( eripcContext,
                                        ERIPC_BUS_SESSION,
                                        DBUS_PATH,
                                        DBUS_SERVICE_SYSTEM_CONTROL,
                                        "startupComplete",
                                        ERIPC_TYPE_STRING, DBUS_APPL_NAME,
                                        ERIPC_TYPE_INT,    my_pid,
                                        ERIPC_TYPE_BOOL,   is_multidoc,
                                        ERIPC_TYPE_STRING, DBUS_SERVICE,
                                        ERIPC_TYPE_INT,    my_xid,
                                        ERIPC_TYPE_BOOL,   show_taskbar,
                                        ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
    } 
}

void ipc_change_filename(const gchar* filename, const gchar* label)
{
    eripc_error_t   result;

    printf("ipc_change_filename(%s, %s)\n", filename, label);
    result = eripc_send_signal_varargs( eripcContext,
                                        ERIPC_BUS_SESSION,
                                        DBUS_PATH,
										DBUS_SERVICE_SYSTEM_CONTROL, 
										"renameTask",
										ERIPC_TYPE_INT, g_xid,
										ERIPC_TYPE_STRING, filename,
										ERIPC_TYPE_STRING, label,
										ERIPC_TYPE_INVALID);
    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
    } 
}


//----------------------------------------------------------------------------
// Popup menu
//----------------------------------------------------------------------------

// show the given menu set
gboolean ipc_menu_show_taskbar( const char *cmd )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    //printf( "entry: cmd [%s]", cmd );
    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setTaskbarShow",
                                 ERIPC_TYPE_STRING, cmd,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}

// add a menu set
gboolean ipc_menu_add_menu( const char *name,
                            const char *group1,
                            const char *group2,
                            const char *group3 )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "addMenu",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, "",
                                 ERIPC_TYPE_STRING, DBUS_SERVICE,
                                 ERIPC_TYPE_STRING, group1,
                                 ERIPC_TYPE_STRING, group2,
                                 ERIPC_TYPE_STRING, group3,
                                 ERIPC_TYPE_STRING, "",
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// add a menu group
gboolean ipc_menu_add_group( const char *name,
                             const char *parent, 
                             const char *image )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "addGroup",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, parent,
                                 ERIPC_TYPE_STRING, "",
                                 ERIPC_TYPE_STRING, image,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// add a menu item
gboolean ipc_menu_add_item( const char *name,
                            const char *parent, 
                            const char *image  )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "addItem",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, parent,
                                 ERIPC_TYPE_STRING, "",
                                 ERIPC_TYPE_STRING, image,
                                 ERIPC_TYPE_INVALID);

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// add a toolbar item
gboolean ipc_menu_add_toolbar_item( const char *menu,
                                    const char *group, 
                                    const char *item  )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "addToolbarItem",
                                 ERIPC_TYPE_STRING, menu,
                                 ERIPC_TYPE_STRING, group,
                                 ERIPC_TYPE_STRING, item,
                                 ERIPC_TYPE_INVALID);

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}

// set text for a menu set
gboolean ipc_menu_set_menu_label ( const char *name,
                                   const char *label )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setMenuLabel",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, label,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// set text for a menu group
gboolean ipc_menu_set_group_label ( const char *name,
                                    const char *label )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setGroupLabel",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, label,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// set text for an menu item
gboolean ipc_menu_set_item_label ( const char *name,
                                   const char *parent, 
                                   const char *label )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setItemLabel",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, parent,
                                 ERIPC_TYPE_STRING, label,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// show the given menu set
gboolean ipc_menu_show_menu( const char *name )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "showMenu",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// remove the given menu set
gboolean ipc_remove_menu( const char *name )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "removeMenu",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS)
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    }

    return ok;
}


// set the state of a menu group
gboolean ipc_menu_set_group_state( const char *name,
                                   const char *state )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setGroupState",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, state,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// set the state of a menu item
gboolean ipc_menu_set_item_state( const char *name,
                                  const char *parent,
                                  const char *state  )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setItemState",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, parent,
                                 ERIPC_TYPE_STRING, state,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}


// show/hide the busy dialog
// TODO: not used, could be used when generation of game or solution takes a lot
//       of time
gboolean ipc_menu_busy_show( gboolean look_busy )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    // broadcast signal over session bus
    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setBusyShow",
                                 ERIPC_TYPE_STRING, look_busy ? "show" : "hide",
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}

// set the state of a menu item
gboolean ipc_menu_set_statusitem_state( const char *name,
                                        const char *state  )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;

    g_assert( eripcContext );

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_POPUP_MENU,
                                 "setStatusItemState",
                                 ERIPC_TYPE_STRING, name,
                                 ERIPC_TYPE_STRING, state,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}

void ipc_ctb_goto(const char *location)
{
    //printf("entry");
    
    eripc_error_t retval;
        
    g_return_if_fail(location != NULL);
    
    eripc_set_timeout(eripcContext, IPC_TIMEOUT_DEFAULT * 1000);
    
    retval = eripc_send_string(eripcContext,
                               NULL, // reply handler
                               NULL, // reply data
                               ERIPC_BUS_SESSION,
                               DBUS_SERVICE_CTB,
                               "gotoLocation",
                               location);

    if (retval != ERIPC_ERROR_SUCCESS) 
    {
        //printf("Error launching eripc handler: %s", eripcErrorStr(retval));
    }
}


// delay sleep mode
gboolean ipc_sys_bg_busy( gboolean look_busy )
{
    gboolean      ok = TRUE;    // return value
    eripc_error_t result;
    char *cmd;

    g_assert( eripcContext );
    
    if (look_busy)
    {
        cmd = "sysSetBgBusy";
    }
    else
    {
        cmd = "sysResetBgBusy";
    }

    result = eripc_send_varargs( eripcContext,
                                 NULL,                  // reply handler
                                 NULL,                  // user data
                                 ERIPC_BUS_SESSION,
                                 DBUS_SERVICE_SYSTEM_CONTROL,
                                 cmd,
                                 ERIPC_TYPE_INVALID );

    if (result != ERIPC_ERROR_SUCCESS) 
    {
        printf("eripc_send_varargs returns [%d]", result);
        ok = FALSE;
    } 

    return ok;
}

int ipc_get_rotation(void)
{
    eripc_error_t retval;
    eripc_event_info_t *info = NULL;
    int rotation = DRR_PORTRAIT;

    retval = eripc_send_varargs_and_wait( eripcContext,
                                          &info,
                                          ERIPC_BUS_SESSION,
                                          DBUS_SERVICE_SYSTEM_CONTROL,
                                          "sysGetOrientation",
                                          ERIPC_TYPE_INVALID);

    if (retval == ERIPC_ERROR_SUCCESS)
    {
        const eripc_arg_t *arg_array = info->args;
        if (arg_array[0].type == ERIPC_TYPE_STRING)
        {
            const char* orientation = arg_array[0].value.s;
            if (orientation)
            {
                if ( 0 == strcasecmp( orientation, "landscape_anticlockwise") )
                {
                    rotation = DRR_ANTICLOCKWISE;
                }
                else if ( 0 == strcasecmp( orientation, "landscape_clockwise") )
                {
                    rotation = DRR_CLOCKWISE;
                }
                else
                {
                    rotation = DRR_PORTRAIT;
                }
            }
        }
    }

    return rotation;
}

//============================================================================
// Local Function Implementation
//============================================================================

//----------------------------------------------------------------------------
// Signal/message handlers
//----------------------------------------------------------------------------

/* @brief Called when a menu items is activated in Popup menu
 *
 * Application (callee) should handle the item depending on the current state.
 */
static void on_menu_item ( eripc_context_t          *context,
                           const eripc_event_info_t *info,
                           void                     *user_data )
{
    const eripc_arg_t *arg_array = info->args;

    if ((arg_array[0].type == ERIPC_TYPE_STRING) && 
        (arg_array[1].type == ERIPC_TYPE_STRING) && 
        (arg_array[2].type == ERIPC_TYPE_STRING) && 
        (arg_array[3].type == ERIPC_TYPE_STRING))
    {
        const char        *item      = arg_array[0].value.s;
        const char        *group     = arg_array[1].value.s;
        const char        *menu      = arg_array[2].value.s;
        const char        *state     = arg_array[3].value.s;
        
        if (item && group && menu && state)
        {
            menu_on_item_activated( item, group, menu, state );
        }
    }
}

/* @brief Called after a window was activated (set to the foreground)
 *
 * Application (callee) should set its context for the given window and set the 
 * Popupmenu menu context.
 */
static void on_window_activated( eripc_context_t          *context,
                                 const eripc_event_info_t *info,
                                 void                     *user_data )
{
    gchar             *msg        = NULL;
    gboolean          result      = FALSE; 
    const eripc_arg_t *arg_array  = info->args;

    if (arg_array[0].type == ERIPC_TYPE_INT)
    {
        gint window = arg_array[0].value.i;
        
        menu_show();

        msg = g_strdup_printf("[Xournal] Window activated: %d\n", window);
        //printf("%s", msg);
        g_free(msg);
        
        result = TRUE;
    }

    // return result to caller
    eripc_reply_bool(context, info->message_id, result);
}


/* @brief Called after a window was deactivated (set to the background)
 *
 * Application (callee) may adapt its context and free resources.
 */  
// TODO: not used, so called when other app gets foreground?
static void on_window_deactivated( eripc_context_t          *context,
                                   const eripc_event_info_t *info,
                                   void                     *user_data )
{
    gchar             *msg        = NULL;
    gboolean          result      = FALSE; 
    const eripc_arg_t *arg_array  = info->args;

    if (arg_array[0].type == ERIPC_TYPE_INT)
    {
        // TODO: Replace implementation

        gint window = arg_array[0].value.i;
        
        msg = g_strdup_printf("[Xournal] Window deactivated: %d\n", window);
        //printf("%s", msg);
        g_free(msg);
        
        result = TRUE;
    }
    
    // return result to caller
    eripc_reply_bool(context, info->message_id, result);
}

/* @brief Called just before unmounting the volume
 *
 * Application must close all its open files on the given volume. Failing to 
 * this signal may result in unsaved data or currupt files.
 */
// TODO: not used, could be used to save any open file? This call will also be 
//       called before the device shutsdown!
static void on_prepare_unmount ( eripc_context_t          *context,
                                 const eripc_event_info_t *info,
                                 void                     *user_data )
{
    gchar             *msg        = NULL;
    const eripc_arg_t *arg_array  = info->args;

    if (arg_array[0].type == ERIPC_TYPE_STRING)
    {
        const char *mountpoint = arg_array[0].value.s;
        if (mountpoint)
        {
			// save to temp file, will be read after restart.
            save_to_temp_file();

            msg = g_strdup_printf("Device to unmount: %s", mountpoint);
            //printf("%s", msg);
            g_free(msg);
        }
    }
}


/* @brief Called just after unmounting the volume
 *
 * Typically an application should have responded to a prior sysPrepareUnmount 
 * signal, but when a device with volumes was removed unexpectedly it may need 
 * to act on this signal.
 */  
static void on_unmounted ( eripc_context_t          *context,
                           const eripc_event_info_t *info,
                           void                     *user_data )
{
    gchar             *msg        = NULL;
    const eripc_arg_t *arg_array  = info->args;

    if (arg_array[0].type == ERIPC_TYPE_STRING)
    {
        const char  *mountpoint = arg_array[0].value.s;
        if (mountpoint)
        {
            // TODO: Replace implementation
            
            msg = g_strdup_printf("Device unmounted: %s", mountpoint);
            //printf("%s", msg);
            g_free(msg);
        }
    }
}

// -- flipbar and orientation -- 

/* @brief Called when the display's orientation has changed
 *
 * Application may need to adapt its screen size, coordinates and/or origin.
 * Possible values: "portrait", "landscape_clockwise", "landscape_anticlockwise"
 */
static void on_changed_orientation( 
        eripc_context_t          *context,
        const eripc_event_info_t *info,
        void                     *user_data )
{
    
    const eripc_arg_t *arg_array = info->args;
    if (arg_array[0].type == ERIPC_TYPE_STRING)
    {
        const char* orientation = arg_array[0].value.s;
        if (orientation)
        {
            if ( 0 == strcasecmp( orientation, "landscape_anticlockwise") )
            {
                ui.rotation = DRR_ANTICLOCKWISE;
            }
            else if ( 0 == strcasecmp( orientation, "landscape_clockwise") )
            {
                ui.rotation = DRR_CLOCKWISE;
            }
            else
            {
                ui.rotation = DRR_PORTRAIT;
            }
        }
    }
    // get DR size
    gdk_window_get_geometry(gdk_get_default_root_window(), NULL, NULL, &ui.display_width, &ui.display_height, NULL);
    printf("\n============SCREEN: %d x %d\n", ui.display_width, ui.display_height);
}


//ERIPC_TYPE_STRING, iname,
//ERIPC_TYPE_STRING, state,
//    item = "statusbar_stylus";
//    statusbar_add_item(      item, "pointer", g_has_stylus, TRUE);
//    statusbar_add_item_state(item, "eraser");
//    statusbar_add_item_state(item, "pan");
//    statusbar_add_item_state(item, "pen");
//    statusbar_add_item_state(item, "zoom");
//    statusbar_add_item_state(item, "lookup");
static void on_status_item_activated(eripc_context_t *context, const eripc_event_info_t *info, void *user_data)
{
    const eripc_arg_t *arg_array = NULL;

    arg_array = info->args;
    
    g_return_if_fail(arg_array[0].type == ERIPC_TYPE_STRING);
    g_return_if_fail(arg_array[1].type == ERIPC_TYPE_STRING);

    // [0] name     string, name of activated status item
    // [1] state    string, name of state
	//printf("name:%s state %s\n", arg_array[0].value.s, arg_array[1].value.s);
	
	if (ui.toolno[ui.cur_mapping] != TOOL_PEN)
	{
		gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON(buttonPen), TRUE);
	}
	else
	{
		gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON(buttonEraser), TRUE);
		ipc_menu_set_statusitem_state("statusbar_stylus", "eraser"); 
	}
}


